@@ -165,8 +165,10 @@ void QuickItemNodeInstance::initialize(const ObjectNodeInstance::Pointer &object
         quickItem()->setParentItem(qobject_cast<QQuickItem*>(nodeInstanceServer()->quickView()->rootObject()));
     }
 
-    if (s_createEffectItem || instanceId() == 0)
-        designerSupport()->refFromEffectItem(quickItem());
+    if (quickItem()->window()) {
+        if (s_createEffectItem || instanceId() == 0)
+            designerSupport()->refFromEffectItem(quickItem());
+    }
 
     ObjectNodeInstance::initialize(objectNodeInstance);
     quickItem()->update();
@@ -364,7 +366,13 @@ QImage QuickItemNodeInstance::renderImage() const
 
     QRectF renderBoundingRect = boundingRect();
 
-    QImage renderImage = designerSupport()->renderImageForItem(quickItem(), renderBoundingRect, renderBoundingRect.size().toSize());
+    QSize size = renderBoundingRect.size().toSize();
+    static double devicePixelRatio = qgetenv("FORMEDITOR_DEVICE_PIXEL_RATIO").toDouble();
+    size *= devicePixelRatio;
+
+    QImage renderImage = designerSupport()->renderImageForItem(quickItem(), renderBoundingRect, size);
+
+    renderImage.setDevicePixelRatio(devicePixelRatio);
 
     return renderImage;
 }
@@ -465,6 +473,8 @@ QRectF QuickItemNodeInstance::boundingRectWithStepChilds(QQuickItem *parentItem)
 {
     QRectF boundingRect = parentItem->boundingRect();
 
+    boundingRect = boundingRect.united(QRectF(QPointF(0, 0), size()));
+
     foreach (QQuickItem *childItem, parentItem->childItems()) {
         if (!nodeInstanceServer()->hasInstanceForObject(childItem)) {
             QRectF transformedRect = childItem->mapRectToItem(parentItem, boundingRectWithStepChilds(childItem));
@@ -582,8 +592,7 @@ void QuickItemNodeInstance::reparent(const ObjectNodeInstance::Pointer &oldParen
 
     if (quickItem()->parentItem()) {
         refresh();
-        if (quickItem()->window())
-            DesignerSupport::updateDirtyNode(quickItem());
+        DesignerSupport::updateDirtyNode(quickItem());
 
         if (instanceIsValidLayoutable(oldParentInstance, oldParentProperty))
             oldParentInstance->refreshLayoutable();
