@@ -45,8 +45,9 @@ class PluginManager;
 class PluginSpec;
 
 namespace Internal {
-class PluginItem;
 class CollectionItem;
+class PluginFilterModel;
+class PluginItem;
 } // Internal
 
 class EXTENSIONSYSTEM_EXPORT PluginView : public QWidget
@@ -59,6 +60,8 @@ public:
 
     PluginSpec *currentPlugin() const;
     void setFilter(const QString &filter);
+    void setShowHidden(bool showHidden);
+    bool isShowingHidden() const;
 
 signals:
     void currentPluginChanged(ExtensionSystem::PluginSpec *spec);
@@ -71,8 +74,8 @@ private:
     bool setPluginsEnabled(const QSet<PluginSpec *> &plugins, bool enable);
 
     Utils::TreeView *m_categoryView;
-    Utils::LeveledTreeModel<Utils::TreeItem, Internal::CollectionItem, Internal::PluginItem> *m_model;
-    QSortFilterProxyModel *m_sortModel;
+    Utils::TreeModel<Utils::TreeItem, Internal::CollectionItem, Internal::PluginItem> *m_model;
+    Internal::PluginFilterModel *m_sortModel;
 
     friend class Internal::CollectionItem;
     friend class Internal::PluginItem;
