@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef QMLJSAST_P_H
-#define QMLJSAST_P_H
+#pragma once
 
 //
 //  W A R N I N G
@@ -95,11 +94,11 @@ namespace QmlJS {
 
 namespace AST {
 
-template <typename _T1, typename _T2>
-_T1 cast(_T2 *ast)
+template <typename T1, typename T2>
+T1 cast(T2 *ast)
 {
-    if (ast && ast->kind == static_cast<_T1>(0)->K)
-        return static_cast<_T1>(ast);
+    if (ast && ast->kind == static_cast<T1>(0)->K)
+        return static_cast<T1>(ast);
 
     return 0;
 }
@@ -224,7 +223,7 @@ public:
     static void accept(Node *node, Visitor *visitor);
 
     inline static void acceptChild(Node *node, Visitor *visitor)
-    { return accept(node, visitor); } // ### remove
+    { accept(node, visitor); } // ### remove
 
     virtual void accept0(Visitor *visitor) = 0;
     virtual SourceLocation firstSourceLocation() const = 0;
@@ -587,6 +586,8 @@ public:
     virtual SourceLocation lastSourceLocation() const
     { return propertyNameToken; }
 
+    virtual QString asString() const = 0;
+
 // attributes
     SourceLocation propertyNameToken;
 };
@@ -594,7 +595,11 @@ public:
 class QML_PARSER_EXPORT PropertyAssignment: public Node
 {
 public:
-    PropertyAssignment() {}
+    PropertyAssignment(PropertyName *n)
+        : name(n)
+    {}
+// attributes
+    PropertyName *name;
 };
 
 class QML_PARSER_EXPORT PropertyAssignmentList: public Node
@@ -642,7 +647,7 @@ public:
     QMLJS_DECLARE_AST_NODE(PropertyNameAndValue)
 
     PropertyNameAndValue(PropertyName *n, ExpressionNode *v)
-        : name(n), value(v)
+        : PropertyAssignment(n), value(v)
     { kind = K; }
 
     virtual void accept0(Visitor *visitor);
@@ -654,7 +659,6 @@ public:
     { return value->lastSourceLocation(); }
 
 // attributes
-    PropertyName *name;
     SourceLocation colonToken;
     ExpressionNode *value;
     SourceLocation commaToken;
@@ -671,11 +675,11 @@ public:
     };
 
     PropertyGetterSetter(PropertyName *n, FunctionBody *b)
-        : type(Getter), name(n), formals(0), functionBody (b)
+        : PropertyAssignment(n), type(Getter), formals(0), functionBody (b)
     { kind = K; }
 
     PropertyGetterSetter(PropertyName *n, FormalParameterList *f, FunctionBody *b)
-        : type(Setter), name(n), formals(f), functionBody (b)
+        : PropertyAssignment(n), type(Setter), formals(f), functionBody (b)
     { kind = K; }
 
     virtual void accept0(Visitor *visitor);
@@ -689,7 +693,6 @@ public:
 // attributes
     Type type;
     SourceLocation getSetToken;
-    PropertyName *name;
     SourceLocation lparenToken;
     FormalParameterList *formals;
     SourceLocation rparenToken;
@@ -708,6 +711,8 @@ public:
 
     virtual void accept0(Visitor *visitor);
 
+    virtual QString asString() const { return id.toString(); }
+
 // attributes
     QStringRef id;
 };
@@ -722,6 +727,8 @@ public:
 
     virtual void accept0(Visitor *visitor);
 
+    virtual QString asString() const { return id.toString(); }
+
 // attributes
     QStringRef id;
 };
@@ -736,6 +743,8 @@ public:
 
     virtual void accept0(Visitor *visitor);
 
+    virtual QString asString() const { return QString::number(id, 'g', 16); }
+
 // attributes
     double id;
 };
@@ -2759,5 +2768,3 @@ public:
 
 
 QT_QML_END_NAMESPACE
-
-#endif
