@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,39 +9,35 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "pathchooser.h"
 
-#include "fancylineedit.h"
 #include "environment.h"
 #include "qtcassert.h"
 
 #include "synchronousprocess.h"
 #include "hostosinfo.h"
+#include "theme/theme.h"
 
 #include <QDebug>
-#include <QDesktopServices>
 #include <QFileDialog>
 #include <QHBoxLayout>
+#include <QMenu>
 #include <QPushButton>
+#include <QStandardPaths>
 
 /*!
     \class Utils::PathChooser
@@ -53,34 +49,24 @@
     This class has some validation logic for embedding into QWizardPage.
 */
 
+static QString appBundleExpandedPath(const QString &path)
+{
+    if (Utils::HostOsInfo::hostOs() == Utils::OsTypeMac && path.endsWith(QLatin1String(".app"))) {
+        // possibly expand to Foo.app/Contents/MacOS/Foo
+        QFileInfo info(path);
+        if (info.isDir()) {
+            QString exePath = path + QLatin1String("/Contents/MacOS/") + info.completeBaseName();
+            if (QFileInfo::exists(exePath))
+                return exePath;
+        }
+    }
+    return path;
+}
+
+Utils::PathChooser::AboutToShowContextMenuHandler Utils::PathChooser::s_aboutToShowContextMenuHandler;
+
 namespace Utils {
 
-// ------------------ PathValidatingLineEdit
-
-class PathValidatingLineEdit : public FancyLineEdit
-{
-public:
-    explicit PathValidatingLineEdit(PathChooser *chooser, QWidget *parent = 0);
-
-protected:
-    virtual bool validate(const QString &value, QString *errorMessage) const;
-
-private:
-    PathChooser *m_chooser;
-};
-
-PathValidatingLineEdit::PathValidatingLineEdit(PathChooser *chooser, QWidget *parent) :
-    FancyLineEdit(parent),
-    m_chooser(chooser)
-{
-    QTC_ASSERT(chooser, return);
-}
-
-bool PathValidatingLineEdit::validate(const QString &value, QString *errorMessage) const
-{
-    return m_chooser->validatePath(value, errorMessage);
-}
-
 // ------------------ BinaryVersionToolTipEventFilter
 // Event filter to be installed on a lineedit used for entering
 // executables, taking the arguments to print the version ('--version').
@@ -144,16 +130,12 @@ QString BinaryVersionToolTipEventFilter::toolVersion(const QString &binary, cons
 {
     if (binary.isEmpty())
         return QString();
-    QProcess proc;
-    proc.start(binary, arguments);
-    if (!proc.waitForStarted())
+    SynchronousProcess proc;
+    proc.setTimeoutS(1);
+    SynchronousProcessResponse response = proc.runBlocking(binary, arguments);
+    if (response.result != SynchronousProcessResponse::Finished)
         return QString();
-    if (!proc.waitForFinished()) {
-        SynchronousProcess::stopProcess(proc);
-        return QString();
-    }
-    return QString::fromLocal8Bit(QByteArray(proc.readAllStandardOutput()
-        + proc.readAllStandardError()));
+    return response.allOutput();
 }
 
 // Extends BinaryVersionToolTipEventFilter to prepend the existing pathchooser
@@ -176,12 +158,12 @@ private:
 class PathChooserPrivate
 {
 public:
-    PathChooserPrivate(PathChooser *chooser);
+    PathChooserPrivate();
 
     QString expandedPath(const QString &path) const;
 
     QHBoxLayout *m_hLayout;
-    PathValidatingLineEdit *m_lineEdit;
+    FancyLineEdit *m_lineEdit;
 
     PathChooser::Kind m_acceptingKind;
     QString m_dialogTitleOverride;
@@ -193,9 +175,9 @@ public:
     QList<QAbstractButton *> m_buttons;
 };
 
-PathChooserPrivate::PathChooserPrivate(PathChooser *chooser) :
+PathChooserPrivate::PathChooserPrivate() :
     m_hLayout(new QHBoxLayout),
-    m_lineEdit(new PathValidatingLineEdit(chooser)),
+    m_lineEdit(new FancyLineEdit),
     m_acceptingKind(PathChooser::ExistingDirectory),
     m_binaryVersionToolTipEventFilter(0)
 {
@@ -212,8 +194,8 @@ QString PathChooserPrivate::expandedPath(const QString &input) const
     switch (m_acceptingKind) {
     case PathChooser::Command:
     case PathChooser::ExistingCommand: {
-        const QString expanded = m_environment.searchInPath(path, QStringList(m_baseDirectory));
-        return expanded.isEmpty() ? path : expanded;
+        const FileName expanded = m_environment.searchInPath(path, QStringList(m_baseDirectory));
+        return expanded.isEmpty() ? path : expanded.toString();
     }
     case PathChooser::Any:
         break;
@@ -230,44 +212,54 @@ QString PathChooserPrivate::expandedPath(const QString &input) const
 
 PathChooser::PathChooser(QWidget *parent) :
     QWidget(parent),
-    d(new PathChooserPrivate(this))
+    d(new PathChooserPrivate)
 {
     d->m_hLayout->setContentsMargins(0, 0, 0, 0);
 
-    connect(d->m_lineEdit, SIGNAL(validReturnPressed()), this, SIGNAL(returnPressed()));
-    connect(d->m_lineEdit, SIGNAL(textChanged(QString)), this, SIGNAL(changed(QString)));
-    connect(d->m_lineEdit, SIGNAL(validChanged()), this, SIGNAL(validChanged()));
-    connect(d->m_lineEdit, SIGNAL(validChanged(bool)), this, SIGNAL(validChanged(bool)));
-    connect(d->m_lineEdit, SIGNAL(editingFinished()), this, SIGNAL(editingFinished()));
-    connect(d->m_lineEdit, SIGNAL(textChanged(QString)), this, SLOT(slotTextChanged()));
+    d->m_lineEdit->setContextMenuPolicy(Qt::CustomContextMenu);
+
+    connect(d->m_lineEdit, &FancyLineEdit::customContextMenuRequested, this, &PathChooser::contextMenuRequested);
+    connect(d->m_lineEdit, &FancyLineEdit::validReturnPressed, this, &PathChooser::returnPressed);
+    connect(d->m_lineEdit, &QLineEdit::textChanged, this,
+            [this] { emit rawPathChanged(rawPath()); });
+    connect(d->m_lineEdit, &FancyLineEdit::validChanged, this, &PathChooser::validChanged);
+    connect(d->m_lineEdit, &QLineEdit::editingFinished, this, &PathChooser::editingFinished);
+    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] { emit pathChanged(path()); });
 
     d->m_lineEdit->setMinimumWidth(120);
+    d->m_lineEdit->setErrorColor(creatorTheme()->color(Theme::TextColorError));
     d->m_hLayout->addWidget(d->m_lineEdit);
     d->m_hLayout->setSizeConstraint(QLayout::SetMinimumSize);
 
-    addButton(browseButtonLabel(), this, SLOT(slotBrowse()));
+    addButton(browseButtonLabel(), this, [this] { slotBrowse(); });
 
     setLayout(d->m_hLayout);
     setFocusProxy(d->m_lineEdit);
     setFocusPolicy(d->m_lineEdit->focusPolicy());
     setEnvironment(Environment::systemEnvironment());
+
+    d->m_lineEdit->setValidationFunction(defaultValidationFunction());
 }
 
 PathChooser::~PathChooser()
 {
+    // Since it is our focusProxy it can receive focus-out and emit the signal
+    // even when the possible ancestor-receiver is in mid of its destruction.
+    disconnect(d->m_lineEdit, &QLineEdit::editingFinished, this, &PathChooser::editingFinished);
+
     delete d;
 }
 
-void PathChooser::addButton(const QString &text, QObject *receiver, const char *slotFunc)
+void PathChooser::addButton(const QString &text, QObject *context, const std::function<void ()> &callback)
 {
-    insertButton(d->m_buttons.count(), text, receiver, slotFunc);
+    insertButton(d->m_buttons.count(), text, context, callback);
 }
 
-void PathChooser::insertButton(int index, const QString &text, QObject *receiver, const char *slotFunc)
+void PathChooser::insertButton(int index, const QString &text, QObject *context, const std::function<void ()> &callback)
 {
-    QPushButton *button = new QPushButton;
+    auto button = new QPushButton;
     button->setText(text);
-    connect(button, SIGNAL(clicked()), receiver, slotFunc);
+    connect(button, &QAbstractButton::clicked, context, callback);
     d->m_hLayout->insertWidget(index + 1/*line edit*/, button);
     d->m_buttons.insert(index, button);
 }
@@ -312,23 +304,42 @@ void PathChooser::setEnvironment(const Environment &env)
     d->m_environment = env;
     if (path() != oldExpand) {
         triggerChanged();
-        emit changed(rawPath());
+        emit rawPathChanged(rawPath());
     }
 }
 
-QString PathChooser::path() const
-{
-    return d->expandedPath(QDir::fromNativeSeparators(d->m_lineEdit->text()));
-}
-
 QString PathChooser::rawPath() const
 {
-    return QDir::fromNativeSeparators(d->m_lineEdit->text());
+    return rawFileName().toString();
+}
+
+QString PathChooser::path() const
+{
+    return fileName().toString();
+}
+
+FileName PathChooser::rawFileName() const
+{
+    return FileName::fromString(QDir::fromNativeSeparators(d->m_lineEdit->text()));
 }
 
 FileName PathChooser::fileName() const
 {
-    return FileName::fromString(path());
+    return FileName::fromUserInput(d->expandedPath(rawFileName().toString()));
+}
+
+// FIXME: try to remove again
+QString PathChooser::expandedDirectory(const QString &input, const Environment &env,
+                                       const QString &baseDir)
+{
+    if (input.isEmpty())
+        return input;
+    const QString path = QDir::cleanPath(env.expandVariables(input));
+    if (path.isEmpty())
+        return path;
+    if (!baseDir.isEmpty() && QFileInfo(path).isRelative())
+        return QFileInfo(baseDir + QLatin1Char('/') + path).absoluteFilePath();
+    return path;
 }
 
 void PathChooser::setPath(const QString &path)
@@ -341,6 +352,16 @@ void PathChooser::setFileName(const FileName &fn)
     d->m_lineEdit->setText(fn.toUserOutput());
 }
 
+void PathChooser::setErrorColor(const QColor &errorColor)
+{
+    d->m_lineEdit->setErrorColor(errorColor);
+}
+
+void PathChooser::setOkColor(const QColor &okColor)
+{
+    d->m_lineEdit->setOkColor(okColor);
+}
+
 bool PathChooser::isReadOnly() const
 {
     return d->m_lineEdit->isReadOnly();
@@ -388,20 +409,13 @@ void PathChooser::slotBrowse()
         newPath = QFileDialog::getOpenFileName(this,
                 makeDialogTitle(tr("Choose Executable")), predefined,
                 d->m_dialogFilter);
-        if (HostOsInfo::hostOs() == OsTypeMac && newPath.endsWith(QLatin1String(".app"))) {
-            // possibly expand to Foo.app/Contents/MacOS/Foo
-            QFileInfo info(newPath);
-            if (info.isDir()) {
-                QString exePath = newPath + QLatin1String("/Contents/MacOS/") + info.completeBaseName();
-                if (QFileInfo(exePath).isExecutable())
-                    newPath = exePath;
-            }
-        }
+        newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::File: // fall through
         newPath = QFileDialog::getOpenFileName(this,
                 makeDialogTitle(tr("Choose File")), predefined,
                 d->m_dialogFilter);
+        newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::SaveFile:
         newPath = QFileDialog::getSaveFileName(this,
@@ -441,9 +455,16 @@ void PathChooser::slotBrowse()
     triggerChanged();
 }
 
-void PathChooser::slotTextChanged()
+void PathChooser::contextMenuRequested(const QPoint &pos)
 {
-    emit pathChanged(path());
+    if (QMenu *menu = d->m_lineEdit->createStandardContextMenu()) {
+        menu->setAttribute(Qt::WA_DeleteOnClose);
+
+        if (s_aboutToShowContextMenuHandler)
+            s_aboutToShowContextMenuHandler(this, menu);
+
+        menu->popup(d->m_lineEdit->mapToGlobal(pos));
+    }
 }
 
 bool PathChooser::isValid() const
@@ -458,11 +479,32 @@ QString PathChooser::errorMessage() const
 
 void PathChooser::triggerChanged()
 {
-    d->m_lineEdit->triggerChanged();
+    d->m_lineEdit->validate();
 }
 
-bool PathChooser::validatePath(const QString &path, QString *errorMessage)
+void PathChooser::setAboutToShowContextMenuHandler(PathChooser::AboutToShowContextMenuHandler handler)
 {
+    s_aboutToShowContextMenuHandler = handler;
+}
+
+QColor PathChooser::errorColor() const
+{
+    return d->m_lineEdit->errorColor();
+}
+
+QColor PathChooser::okColor() const
+{
+    return d->m_lineEdit->okColor();
+}
+
+FancyLineEdit::ValidationFunction PathChooser::defaultValidationFunction() const
+{
+    return std::bind(&PathChooser::validatePath, this, std::placeholders::_1, std::placeholders::_2);
+}
+
+bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
+{
+    const QString path = edit->text();
     QString expandedPath = d->expandedPath(path);
 
     if (path.isEmpty()) {
@@ -498,6 +540,11 @@ bool PathChooser::validatePath(const QString &path, QString *errorMessage)
                 *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
             return false;
         }
+        if (!fi.isFile()) {
+            if (errorMessage)
+                *errorMessage = tr("The path \"%1\" is not a file.").arg(QDir::toNativeSeparators(expandedPath));
+            return false;
+        }
         break;
     case PathChooser::SaveFile:
         if (!fi.absoluteDir().exists()) {
@@ -505,6 +552,11 @@ bool PathChooser::validatePath(const QString &path, QString *errorMessage)
                 *errorMessage = tr("The directory \"%1\" does not exist.").arg(QDir::toNativeSeparators(fi.absolutePath()));
             return false;
         }
+        if (fi.exists() && fi.isDir()) {
+            if (errorMessage)
+                *errorMessage = tr("The path \"%1\" is not a file.").arg(QDir::toNativeSeparators(fi.absolutePath()));
+            return false;
+        }
         break;
     case PathChooser::ExistingCommand:
         if (!fi.exists()) {
@@ -512,9 +564,9 @@ bool PathChooser::validatePath(const QString &path, QString *errorMessage)
                 *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
             return false;
         }
-        if (!fi.isExecutable()) {
+        if (!fi.isFile() || !fi.isExecutable()) {
             if (errorMessage)
-                *errorMessage = tr("Cannot execute \"%1\".").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not an executable file.").arg(QDir::toNativeSeparators(expandedPath));
             return false;
         }
         break;
@@ -537,53 +589,16 @@ bool PathChooser::validatePath(const QString &path, QString *errorMessage)
         ;
     }
 
-    // Check expected kind
-    switch (d->m_acceptingKind) {
-    case PathChooser::ExistingDirectory:
-        if (!fi.isDir()) {
-            if (errorMessage)
-                *errorMessage = tr("The path <b>%1</b> is not a directory.").arg(QDir::toNativeSeparators(expandedPath));
-            return false;
-        }
-        break;
-
-    case PathChooser::File:
-        if (!fi.isFile()) {
-            if (errorMessage)
-                *errorMessage = tr("The path <b>%1</b> is not a file.").arg(QDir::toNativeSeparators(expandedPath));
-            return false;
-        }
-        break;
-
-    case PathChooser::SaveFile:
-        if (fi.exists() && fi.isDir()) {
-            if (errorMessage)
-                *errorMessage = tr("The path <b>%1</b> is not a file.").arg(QDir::toNativeSeparators(fi.absolutePath()));
-            return false;
-        }
-        break;
-
-    case PathChooser::ExistingCommand:
-        if (!fi.isFile() || !fi.isExecutable()) {
-            if (errorMessage)
-                *errorMessage = tr("The path <b>%1</b> is not an executable file.").arg(QDir::toNativeSeparators(expandedPath));
-            return false;
-        }
-
-    case PathChooser::Command:
-        break;
-
-    case PathChooser::Any:
-        break;
-
-    default:
-        ;
-    }
     if (errorMessage)
-        *errorMessage = tr("Full path: <b>%1</b>").arg(QDir::toNativeSeparators(expandedPath));
+        *errorMessage = tr("Full path: \"%1\"").arg(QDir::toNativeSeparators(expandedPath));
     return true;
 }
 
+void PathChooser::setValidationFunction(const FancyLineEdit::ValidationFunction &fn)
+{
+    d->m_lineEdit->setValidationFunction(fn);
+}
+
 QString PathChooser::label()
 {
     return tr("Path:");
@@ -595,7 +610,7 @@ QString PathChooser::homePath()
     // does not let people actually display the contents of their home
     // directory. Alternatively, create a QtCreator-specific directory?
     if (HostOsInfo::isWindowsHost())
-        return QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
+        return QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
     return QDir::homePath();
 }
 
@@ -604,7 +619,7 @@ void PathChooser::setExpectedKind(Kind expected)
     if (d->m_acceptingKind == expected)
         return;
     d->m_acceptingKind = expected;
-    d->m_lineEdit->triggerChanged();
+    d->m_lineEdit->validate();
 }
 
 PathChooser::Kind PathChooser::expectedKind() const
@@ -664,9 +679,9 @@ void PathChooser::installLineEditVersionToolTip(QLineEdit *le, const QStringList
     ef->setArguments(arguments);
 }
 
-void PathChooser::setHistoryCompleter(const QString &historyKey)
+void PathChooser::setHistoryCompleter(const QString &historyKey, bool restoreLastItemFromHistory)
 {
-    d->m_lineEdit->setHistoryCompleter(historyKey);
+    d->m_lineEdit->setHistoryCompleter(historyKey, restoreLastItemFromHistory);
 }
 
 QStringList PathChooser::commandVersionArguments() const
