@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef AVDDIALOG_H
-#define AVDDIALOG_H
+#pragma once
 
 #include "ui_addnewavddialog.h"
 
@@ -33,6 +32,7 @@
 
 namespace Android {
 class AndroidConfig;
+class SdkPlatform;
 
 namespace Internal {
 
@@ -43,15 +43,14 @@ public:
     explicit AvdDialog(int minApiLevel, const QString &targetArch,
                        const AndroidConfig *config, QWidget *parent = 0);
 
-    QString target() const;
+    Android::SdkPlatform target() const;
     QString name() const;
     QString abi() const;
     int sdcardSize() const;
     bool isValid() const;
 
-private slots:
-    void updateApiLevelComboBox();
 private:
+    void updateApiLevelComboBox();
     bool eventFilter(QObject *obj, QEvent *event);
 
     Ui::AddNewAVDDialog m_avdDialog;
@@ -62,5 +61,3 @@ private:
 };
 }
 }
-
-#endif // AVDDIALOG_H
