@@ -28,7 +28,9 @@
 #include "quicktestvisitors.h"
 #include "quicktest_utils.h"
 #include "../autotest_utils.h"
+#include "../testcodeparser.h"
 
+#include <projectexplorer/session.h>
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/qmljsdialect.h>
 #include <qmljstools/qmljsmodelmanager.h>
@@ -41,8 +43,15 @@ namespace Internal {
 TestTreeItem *QuickTestParseResult::createTestTreeItem() const
 {
     if (itemType == TestTreeItem::Root || itemType == TestTreeItem::TestDataTag)
-        return 0;
-    return QuickTestTreeItem::createTestItem(this);
+        return nullptr;
+
+    QuickTestTreeItem *item = new QuickTestTreeItem(name, fileName, itemType);
+    item->setProFile(proFile);
+    item->setLine(line);
+    item->setColumn(column);
+    for (const TestParseResult *funcResult : children)
+        item->appendChild(funcResult->createTestTreeItem());
+    return item;
 }
 
 static bool includesQtQuickTest(const CPlusPlus::Document::Ptr &doc,
@@ -50,26 +59,25 @@ static bool includesQtQuickTest(const CPlusPlus::Document::Ptr &doc,
 {
     static QStringList expectedHeaderPrefixes
             = Utils::HostOsInfo::isMacHost()
-            ? QStringList({ QLatin1String("QtQuickTest.framework/Headers"),
-                            QLatin1String("QtQuickTest") })
-            : QStringList({ QLatin1String("QtQuickTest") });
+            ? QStringList({"QtQuickTest.framework/Headers", "QtQuickTest"})
+            : QStringList({"QtQuickTest"});
 
     const QList<CPlusPlus::Document::Include> includes = doc->resolvedIncludes();
 
-    foreach (const CPlusPlus::Document::Include &inc, includes) {
-        if (inc.unresolvedFileName() == QLatin1String("QtQuickTest/quicktest.h")) {
-            foreach (const QString &prefix, expectedHeaderPrefixes) {
+    for (const CPlusPlus::Document::Include &inc : includes) {
+        if (inc.unresolvedFileName() == "QtQuickTest/quicktest.h") {
+            for (const QString &prefix : expectedHeaderPrefixes) {
                 if (inc.resolvedFileName().endsWith(
-                            QString::fromLatin1("%1/quicktest.h").arg(prefix))) {
+                            QString("%1/quicktest.h").arg(prefix))) {
                     return true;
                 }
             }
         }
     }
 
-    foreach (const QString &include, snapshot.allIncludesForDocument(doc->fileName())) {
-        foreach (const QString &prefix, expectedHeaderPrefixes) {
-            if (include.endsWith(QString::fromLatin1("%1/quicktest.h").arg(prefix)))
+    for (const QString &include : snapshot.allIncludesForDocument(doc->fileName())) {
+        for (const QString &prefix : expectedHeaderPrefixes) {
+            if (include.endsWith(QString("%1/quicktest.h").arg(prefix)))
                 return true;
         }
     }
@@ -83,7 +91,7 @@ static QString quickTestSrcDir(const CppTools::CppModelManager *cppMM,
     const QList<CppTools::ProjectPart::Ptr> parts = cppMM->projectPart(fileName);
     if (parts.size() > 0) {
         QByteArray projDefines(parts.at(0)->projectDefines);
-        foreach (const QByteArray &line, projDefines.split('\n')) {
+        for (const QByteArray &line : projDefines.split('\n')) {
             if (line.contains(qtsd)) {
                 QByteArray result = line.mid(line.indexOf(qtsd) + qtsd.length());
                 if (result.startsWith('"'))
@@ -101,7 +109,7 @@ static QString quickTestName(const CPlusPlus::Document::Ptr &doc)
 {
     const QList<CPlusPlus::Document::MacroUse> macros = doc->macroUses();
 
-    foreach (const CPlusPlus::Document::MacroUse &macro, macros) {
+    for (const CPlusPlus::Document::MacroUse &macro : macros) {
         if (!macro.isFunctionLike())
             continue;
         const QByteArray name = macro.macro().name();
@@ -114,7 +122,7 @@ static QString quickTestName(const CPlusPlus::Document::Ptr &doc)
     return QString();
 }
 
-static QList<QmlJS::Document::Ptr> scanDirectoryForQuickTestQmlFiles(const QString &srcDir)
+QList<QmlJS::Document::Ptr> QuickTestParser::scanDirectoryForQuickTestQmlFiles(const QString &srcDir) const
 {
     QStringList dirs(srcDir);
     QmlJS::ModelManagerInterface *qmlJsMM = QmlJSTools::Internal::ModelManager::instance();
@@ -122,25 +130,29 @@ static QList<QmlJS::Document::Ptr> scanDirectoryForQuickTestQmlFiles(const QStri
     QFutureInterface<void> future;
     QmlJS::PathsAndLanguages paths;
     paths.maybeInsert(Utils::FileName::fromString(srcDir), QmlJS::Dialect::Qml);
-    const bool emitDocumentChanges = false;
-    const bool onlyTheLib = false;
     QmlJS::ModelManagerInterface::importScan(future, qmlJsMM->workingCopy(), paths, qmlJsMM,
-        emitDocumentChanges, onlyTheLib);
+        false /*emitDocumentChanges*/, false /*onlyTheLib*/, true /*forceRescan*/ );
 
     const QmlJS::Snapshot snapshot = QmlJSTools::Internal::ModelManager::instance()->snapshot();
     QDirIterator it(srcDir, QDir::Dirs | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
     while (it.hasNext()) {
         it.next();
         QFileInfo fi(it.fileInfo().canonicalFilePath());
-        dirs << fi.filePath();
+        dirs.append(fi.filePath());
     }
+    emit updateWatchPaths(dirs);
+
     QList<QmlJS::Document::Ptr> foundDocs;
 
-    foreach (const QString &path, dirs) {
+    for (const QString &path : dirs) {
         const QList<QmlJS::Document::Ptr> docs = snapshot.documentsInDirectory(path);
-        foreach (const QmlJS::Document::Ptr &doc, docs) {
-            const QString fileName(QFileInfo(doc->fileName()).fileName());
-            if (fileName.startsWith(QLatin1String("tst_")) && fileName.endsWith(QLatin1String(".qml")))
+        for (const QmlJS::Document::Ptr &doc : docs) {
+            const QFileInfo fi(doc->fileName());
+            // using working copy above might provide no more existing files
+            if (!fi.exists())
+                continue;
+            const QString fileName(fi.fileName());
+            if (fileName.startsWith("tst_") && fileName.endsWith(".qml"))
                 foundDocs << doc;
         }
     }
@@ -157,8 +169,11 @@ static bool checkQmlDocumentForQuickTestCode(QFutureInterface<TestParseResultPtr
         return false;
     QmlJS::AST::Node *ast = qmlJSDoc->ast();
     QTC_ASSERT(ast, return false);
-    TestQmlVisitor qmlVisitor(qmlJSDoc);
+    QmlJS::Snapshot snapshot = QmlJS::ModelManagerInterface::instance()->snapshot();
+    TestQmlVisitor qmlVisitor(qmlJSDoc, snapshot);
     QmlJS::AST::Node::accept(ast, &qmlVisitor);
+    if (!qmlVisitor.isValid())
+        return false;
 
     const QString testCaseName = qmlVisitor.testCaseName();
     const TestCodeLocationAndType tcLocationAndType = qmlVisitor.testCaseLocation();
@@ -192,9 +207,9 @@ static bool checkQmlDocumentForQuickTestCode(QFutureInterface<TestParseResultPtr
     return true;
 }
 
-static bool handleQtQuickTest(QFutureInterface<TestParseResultPtr> futureInterface,
-                              CPlusPlus::Document::Ptr document,
-                              const Core::Id &id)
+bool QuickTestParser::handleQtQuickTest(QFutureInterface<TestParseResultPtr> futureInterface,
+                                        CPlusPlus::Document::Ptr document,
+                                        const Core::Id &id) const
 {
     const CppTools::CppModelManager *modelManager = CppTools::CppModelManager::instance();
     if (quickTestName(document).isEmpty())
@@ -210,13 +225,37 @@ static bool handleQtQuickTest(QFutureInterface<TestParseResultPtr> futureInterfa
     if (srcDir.isEmpty())
         return false;
 
+    if (futureInterface.isCanceled())
+        return false;
     const QList<QmlJS::Document::Ptr> qmlDocs = scanDirectoryForQuickTestQmlFiles(srcDir);
     bool result = false;
-    foreach (const QmlJS::Document::Ptr &qmlJSDoc, qmlDocs)
+    for (const QmlJS::Document::Ptr &qmlJSDoc : qmlDocs) {
+        if (futureInterface.isCanceled())
+            break;
         result |= checkQmlDocumentForQuickTestCode(futureInterface, qmlJSDoc, id, proFile);
+    }
     return result;
 }
 
+QuickTestParser::QuickTestParser()
+    : CppParser()
+{
+    connect(ProjectExplorer::SessionManager::instance(),
+            &ProjectExplorer::SessionManager::startupProjectChanged, [this] {
+        const QStringList &dirs = m_directoryWatcher.directories();
+        if (!dirs.isEmpty())
+            m_directoryWatcher.removePaths(dirs);
+    });
+    connect(&m_directoryWatcher, &QFileSystemWatcher::directoryChanged,
+                     [this] { TestTreeModel::instance()->parser()->emitUpdateTestTree(this); });
+    connect(this, &QuickTestParser::updateWatchPaths,
+            &m_directoryWatcher, &QFileSystemWatcher::addPaths, Qt::QueuedConnection);
+}
+
+QuickTestParser::~QuickTestParser()
+{
+}
+
 void QuickTestParser::init(const QStringList &filesToParse)
 {
     m_qmlSnapshot = QmlJSTools::Internal::ModelManager::instance()->snapshot();
