@@ -1,7 +1,7 @@
-/**************************************************************************
+/****************************************************************************
 **
-** Copyright (C) 2015 Lorenz Haas
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 Lorenz Haas
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -38,15 +33,13 @@
 
 #include "../beautifierconstants.h"
 #include "../beautifierplugin.h"
-#include "../command.h"
 
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
-#include <coreplugin/icontext.h>
-#include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 #include <cppeditor/cppeditorconstants.h>
 #include <projectexplorer/projecttree.h>
@@ -76,7 +69,7 @@ ArtisticStyle::~ArtisticStyle()
 bool ArtisticStyle::initialize()
 {
     Core::ActionContainer *menu = Core::ActionManager::createMenu(Constants::ArtisticStyle::MENU_ID);
-    menu->menu()->setTitle(QLatin1String(Constants::ArtisticStyle::DISPLAY_NAME));
+    menu->menu()->setTitle(tr(Constants::ArtisticStyle::DISPLAY_NAME));
 
     m_formatFile = new QAction(BeautifierPlugin::msgFormatCurrentFile(), this);
     menu->addAction(Core::ActionManager::registerAction(m_formatFile,
@@ -85,75 +78,100 @@ bool ArtisticStyle::initialize()
 
     Core::ActionManager::actionContainer(Constants::MENU_ID)->addMenu(menu);
 
+    connect(m_settings, &ArtisticStyleSettings::supportedMimeTypesChanged,
+            [this] { updateActions(Core::EditorManager::currentEditor()); });
+
     return true;
 }
 
+QString ArtisticStyle::id() const
+{
+    return QLatin1String(Constants::ArtisticStyle::DISPLAY_NAME);
+}
+
 void ArtisticStyle::updateActions(Core::IEditor *editor)
 {
-    m_formatFile->setEnabled(editor && editor->document()->id() == CppEditor::Constants::CPPEDITOR_ID);
+    m_formatFile->setEnabled(editor && m_settings->isApplicable(editor->document()));
 }
 
 QList<QObject *> ArtisticStyle::autoReleaseObjects()
 {
-    ArtisticStyleOptionsPage *optionsPage = new ArtisticStyleOptionsPage(m_settings, this);
-    return QList<QObject *>() << optionsPage;
+    return {new ArtisticStyleOptionsPage(m_settings, this)};
 }
 
 void ArtisticStyle::formatFile()
 {
-    QString cfgFileName;
+    const QString cfgFileName = configurationFile();
+    if (cfgFileName.isEmpty()) {
+        BeautifierPlugin::showError(BeautifierPlugin::msgCannotGetConfigurationFile(
+                                        tr(Constants::ArtisticStyle::DISPLAY_NAME)));
+    } else {
+        m_beautifierPlugin->formatCurrentFile(command(cfgFileName));
+    }
+}
+
+QString ArtisticStyle::configurationFile() const
+{
+    if (m_settings->useCustomStyle())
+        return m_settings->styleFileName(m_settings->customStyle());
 
     if (m_settings->useOtherFiles()) {
         if (const ProjectExplorer::Project *project
                 = ProjectExplorer::ProjectTree::currentProject()) {
             const QStringList files = project->files(ProjectExplorer::Project::AllFiles);
-            for (int i = 0, total = files.size(); i < total; ++i) {
-                const QString &file = files.at(i);
-                if (!file.endsWith(QLatin1String(".astylerc")))
+            for (const QString &file : files) {
+                if (!file.endsWith(".astylerc"))
                     continue;
                 const QFileInfo fi(file);
-                if (fi.isReadable()) {
-                    cfgFileName = file;
-                    break;
-                }
+                if (fi.isReadable())
+                    return file;
             }
         }
     }
 
-    if (cfgFileName.isEmpty() && m_settings->useHomeFile()) {
+    if (m_settings->useHomeFile()) {
         const QDir homeDirectory = QDir::home();
-        QString file = homeDirectory.filePath(QLatin1String(".astylerc"));
-        if (QFile::exists(file)) {
-            cfgFileName = file;
-        } else {
-            file = homeDirectory.filePath(QLatin1String("astylerc"));
-            if (QFile::exists(file))
-                cfgFileName = file;
-        }
+        QString file = homeDirectory.filePath(".astylerc");
+        if (QFile::exists(file))
+            return file;
+        file = homeDirectory.filePath("astylerc");
+        if (QFile::exists(file))
+            return file;
     }
 
-    if (m_settings->useCustomStyle())
-        cfgFileName = m_settings->styleFileName(m_settings->customStyle());
+    return QString();
+}
 
-    if (cfgFileName.isEmpty()) {
-        BeautifierPlugin::showError(BeautifierPlugin::msgCannotGetConfigurationFile(
-                                        QLatin1String(Constants::ArtisticStyle::DISPLAY_NAME)));
-    } else {
-        Command command;
-        command.setExecutable(m_settings->command());
-        command.addOption(QLatin1String("-q"));
-        command.addOption(QLatin1String("--options=") + cfgFileName);
+Command ArtisticStyle::command() const
+{
+    const QString cfgFile = configurationFile();
+    return cfgFile.isEmpty() ? Command() : command(cfgFile);
+}
 
-        if (m_settings->version() > ArtisticStyleSettings::Version_2_03) {
-            command.setProcessing(Command::PipeProcessing);
+bool ArtisticStyle::isApplicable(const Core::IDocument *document) const
+{
+    return m_settings->isApplicable(document);
+}
+
+Command ArtisticStyle::command(const QString &cfgFile) const
+{
+    Command command;
+    command.setExecutable(m_settings->command());
+    command.addOption("-q");
+    command.addOption("--options=" + cfgFile);
+
+    const int version = m_settings->version();
+    if (version > ArtisticStyleSettings::Version_2_03) {
+        command.setProcessing(Command::PipeProcessing);
+        if (version == ArtisticStyleSettings::Version_2_04)
             command.setPipeAddsNewline(true);
-            command.setReturnsCRLF(Utils::HostOsInfo::isWindowsHost());
-        } else {
-            command.addOption(QLatin1String("%file"));
-        }
-
-        m_beautifierPlugin->formatCurrentFile(command);
+        command.setReturnsCRLF(Utils::HostOsInfo::isWindowsHost());
+        command.addOption("-z2");
+    } else {
+        command.addOption("%file");
     }
+
+    return command;
 }
 
 } // namespace ArtisticStyle
