@@ -92,6 +92,17 @@ public:
     virtual QString displayName() const = 0;
     virtual Id id() const = 0;
 
+    /*!
+     * \brief isVcsFileOrDirectory
+     * \param fileName
+     * \return True if filename is a file or directory that is maintained by the
+     * version control system.
+     *
+     * It will return true only for exact matches of the name, not for e.g. files in a
+     * directory owned by the version control system (e.g. .git/control).
+     */
+    virtual bool isVcsFileOrDirectory(const Utils::FileName &fileName) const = 0;
+
     /*!
      * Returns whether files in this directory should be managed with this
      * version control.
@@ -232,6 +243,9 @@ public:
     { }
     ~TestVersionControl();
 
+    bool isVcsFileOrDirectory(const Utils::FileName &fileName) const final
+    { Q_UNUSED(fileName); return false; }
+
     void setManagedDirectories(const QHash<QString, QString> &dirs);
     void setManagedFiles(const QSet<QString> &files);
 
