@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef DEBUGGERMAINWINDOW_H
-#define DEBUGGERMAINWINDOW_H
+#pragma once
 
 #include "debugger_global.h"
 
@@ -41,7 +40,7 @@ class QComboBox;
 class QStackedWidget;
 QT_END_NAMESPACE
 
-namespace Core { class IMode; }
+namespace Core { class Id; }
 
 namespace Utils {
 
@@ -63,26 +62,39 @@ public:
         QByteArray dockId;
         QPointer<QWidget> widget;
         QByteArray anchorDockId;
-        OperationType operationType;
-        bool visibleByDefault;
-        Qt::DockWidgetArea area;
+        OperationType operationType = Raise;
+        bool visibleByDefault = true;
+        Qt::DockWidgetArea area = Qt::BottomDockWidgetArea;
     };
 
     Perspective() = default;
-    Perspective(const QString &name, const QVector<Operation> &operations);
+    // Takes ownership of \a centralWidget and all dock widgets in \a operations.
+    Perspective(const QString &name, const QVector<Operation> &operations,
+                QWidget *centralWidget = 0);
+    ~Perspective();
 
     void addOperation(const Operation &operation);
 
     QVector<Operation> operations() const { return m_operations; }
     QVector<QByteArray> docks() const { return m_docks; }
+    QWidget *centralWidget() const { return m_centralWidget; }
 
     QString name() const;
     void setName(const QString &name);
 
+    using Callback = std::function<void()>;
+    void setAboutToActivateCallback(const Callback &cb);
+    void aboutToActivate() const;
+
 private:
+    Perspective(const Perspective &) = delete;
+    void operator=(const Perspective &) = delete;
+
     QString m_name;
     QVector<QByteArray> m_docks;
     QVector<Operation> m_operations;
+    QPointer<QWidget> m_centralWidget;
+    Callback m_aboutToActivateCallback;
 };
 
 class DEBUGGER_EXPORT ToolbarDescription
@@ -93,7 +105,7 @@ public:
 
     QList<QWidget *> widgets() const;
 
-    void addAction(QAction *action);
+    void addAction(QAction *action, const QIcon &toolbarIcon = QIcon());
     void addWidget(QWidget *widget);
 
 private:
@@ -108,7 +120,7 @@ public:
     DebuggerMainWindow();
     ~DebuggerMainWindow() override;
 
-    void registerPerspective(const QByteArray &perspectiveId, const Perspective &perspective);
+    void registerPerspective(const QByteArray &perspectiveId, const Perspective *perspective);
     void registerToolbar(const QByteArray &perspectiveId, QWidget *widget);
 
     void saveCurrentPerspective();
@@ -120,6 +132,9 @@ public:
     void showStatusMessage(const QString &message, int timeoutMS);
     QDockWidget *dockWidget(const QByteArray &dockId) const;
     QByteArray currentPerspective() const { return m_currentPerspectiveId; }
+    QStackedWidget *centralWidgetStack() const { return m_centralWidgetStack; }
+
+    void onModeChanged(Core::Id mode);
 
 private:
     QDockWidget *registerDockWidget(const QByteArray &dockId, QWidget *widget);
@@ -128,16 +143,16 @@ private:
     QByteArray m_currentPerspectiveId;
     QComboBox *m_perspectiveChooser;
     QStackedWidget *m_controlsStackWidget;
+    QStackedWidget *m_centralWidgetStack;
+    QWidget *m_editorPlaceHolder;
     Utils::StatusLabel *m_statusLabel;
-    QDockWidget *m_toolbarDock;
+    QDockWidget *m_toolbarDock = nullptr;
 
     QHash<QByteArray, QDockWidget *> m_dockForDockId;
     QHash<QByteArray, QWidget *> m_toolbarForPerspectiveId;
-    QHash<QByteArray, Perspective> m_perspectiveForPerspectiveId;
+    QHash<QByteArray, const Perspective *> m_perspectiveForPerspectiveId;
 };
 
-QWidget *createModeWindow(Core::IMode *mode, DebuggerMainWindow *mainWindow, QWidget *central);
+DEBUGGER_EXPORT QWidget *createModeWindow(const Core::Id &mode, DebuggerMainWindow *mainWindow);
 
 } // Utils
-
-#endif // DEBUGGERMAINWINDOW_H
