@@ -292,8 +292,8 @@ GdbOptionsPageWidget::GdbOptionsPageWidget()
 
     //lineEditSelectedPluginBreakpointsPattern->
     //    setEnabled(action(SelectedPluginBreakpoints)->value().toBool());
-    //connect(radioButtonSelectedPluginBreakpoints, SIGNAL(toggled(bool)),
-    //    lineEditSelectedPluginBreakpointsPattern, SLOT(setEnabled(bool)));
+    //connect(radioButtonSelectedPluginBreakpoints, &QRadioButton::toggled,
+    //    lineEditSelectedPluginBreakpointsPattern, &QLineEdit::setEnabled);
 }
 
 GdbOptionsPage::GdbOptionsPage()
@@ -302,7 +302,7 @@ GdbOptionsPage::GdbOptionsPage()
     setDisplayName(tr("GDB"));
     setCategory(Constants::DEBUGGER_SETTINGS_CATEGORY);
     setDisplayCategory(QCoreApplication::translate("Debugger", Constants::DEBUGGER_SETTINGS_TR_CATEGORY));
-    setCategoryIcon(QLatin1String(Constants::DEBUGGER_COMMON_SETTINGS_CATEGORY_ICON));
+    setCategoryIcon(Utils::Icon(Constants::DEBUGGER_COMMON_SETTINGS_CATEGORY_ICON));
 }
 
 QWidget *GdbOptionsPage::widget()
@@ -392,13 +392,6 @@ GdbOptionsPageWidget2::GdbOptionsPageWidget2()
            "calls and is very likely to destroy your debugging session.</p></body></html>"));
     }
 
-    auto checkBoxAttemptQuickStart = new QCheckBox(groupBoxDangerous);
-    checkBoxAttemptQuickStart->setText(GdbOptionsPage::tr("Attempt quick start"));
-    checkBoxAttemptQuickStart->setToolTip(GdbOptionsPage::tr(
-        "<html><head/><body>Postpones reading debug information as long as possible. "
-        "This can result in faster startup times at the price of not being able to "
-        "set breakpoints by file and number.</body></html>"));
-
     auto checkBoxMultiInferior = new QCheckBox(groupBoxDangerous);
     checkBoxMultiInferior->setText(GdbOptionsPage::tr("Debug all children"));
     checkBoxMultiInferior->setToolTip(GdbOptionsPage::tr(
@@ -415,7 +408,6 @@ GdbOptionsPageWidget2::GdbOptionsPageWidget2()
     formLayout->addRow(checkBoxBreakOnAbort);
     if (checkBoxEnableReverseDebugging)
         formLayout->addRow(checkBoxEnableReverseDebugging);
-    formLayout->addRow(checkBoxAttemptQuickStart);
     formLayout->addRow(checkBoxMultiInferior);
 
     auto gridLayout = new QGridLayout(this);
@@ -426,7 +418,6 @@ GdbOptionsPageWidget2::GdbOptionsPageWidget2()
     group.insert(action(BreakOnWarning), checkBoxBreakOnWarning);
     group.insert(action(BreakOnFatal), checkBoxBreakOnFatal);
     group.insert(action(BreakOnAbort), checkBoxBreakOnAbort);
-    group.insert(action(AttemptQuickStart), checkBoxAttemptQuickStart);
     group.insert(action(MultiInferior), checkBoxMultiInferior);
     if (checkBoxEnableReverseDebugging)
         group.insert(action(EnableReverseDebugging), checkBoxEnableReverseDebugging);
@@ -453,7 +444,7 @@ GdbOptionsPage2::GdbOptionsPage2()
     setDisplayName(tr("GDB Extended"));
     setCategory(Constants::DEBUGGER_SETTINGS_CATEGORY);
     setDisplayCategory(QCoreApplication::translate("Debugger", Constants::DEBUGGER_SETTINGS_TR_CATEGORY));
-    setCategoryIcon(QLatin1String(Constants::DEBUGGER_COMMON_SETTINGS_CATEGORY_ICON));
+    setCategoryIcon(Utils::Icon(Constants::DEBUGGER_COMMON_SETTINGS_CATEGORY_ICON));
 }
 
 QWidget *GdbOptionsPage2::widget()
