@@ -148,13 +148,14 @@ void DeviceManager::load()
 
     Utils::PersistentSettingsReader reader;
     // read devices file from global settings path
+    QHash<Core::Id, Core::Id> defaultDevices;
     QList<IDevice::Ptr> sdkDevices;
     if (reader.load(systemSettingsFilePath(QLatin1String("/qtcreator/devices.xml"))))
-        sdkDevices = fromMap(reader.restoreValues().value(QLatin1String(DeviceManagerKey)).toMap());
+        sdkDevices = fromMap(reader.restoreValues().value(DeviceManagerKey).toMap(), &defaultDevices);
     // read devices file from user settings path
     QList<IDevice::Ptr> userDevices;
     if (reader.load(settingsFilePath(QLatin1String("/devices.xml"))))
-        userDevices = fromMap(reader.restoreValues().value(QLatin1String(DeviceManagerKey)).toMap());
+        userDevices = fromMap(reader.restoreValues().value(DeviceManagerKey).toMap(), &defaultDevices);
     // Insert devices into the model. Prefer the higher device version when there are multiple
     // devices with the same id.
     foreach (IDevice::Ptr device, userDevices) {
@@ -172,18 +173,25 @@ void DeviceManager::load()
     foreach (const IDevice::Ptr &sdkDevice, sdkDevices)
         addDevice(sdkDevice);
 
-    ensureOneDefaultDevicePerType();
+    // Overwrite with the saved default devices.
+    for (auto itr = defaultDevices.constBegin(); itr != defaultDevices.constEnd(); ++itr) {
+        IDevice::ConstPtr device = find(itr.value());
+        if (device)
+            d->defaultDevices[device->type()] = device->id();
+    }
 
     emit devicesLoaded();
 }
 
-QList<IDevice::Ptr> DeviceManager::fromMap(const QVariantMap &map)
+QList<IDevice::Ptr> DeviceManager::fromMap(const QVariantMap &map,
+                                           QHash<Core::Id, Core::Id> *defaultDevices)
 {
     QList<IDevice::Ptr> devices;
-    const QVariantMap defaultDevsMap = map.value(QLatin1String(DefaultDevicesKey)).toMap();
-    for (QVariantMap::ConstIterator it = defaultDevsMap.constBegin();
-         it != defaultDevsMap.constEnd(); ++it) {
-        d->defaultDevices.insert(Core::Id::fromString(it.key()), Core::Id::fromSetting(it.value()));
+
+    if (defaultDevices) {
+        const QVariantMap defaultDevsMap = map.value(DefaultDevicesKey).toMap();
+        for (auto it = defaultDevsMap.constBegin(); it != defaultDevsMap.constEnd(); ++it)
+            defaultDevices->insert(Core::Id::fromString(it.key()), Core::Id::fromSetting(it.value()));
     }
     const QVariantList deviceList = map.value(QLatin1String(DeviceListKey)).toList();
     foreach (const QVariant &v, deviceList) {
@@ -349,7 +357,7 @@ DeviceManager::DeviceManager(bool isInstance) : d(new DeviceManagerPrivate)
         m_instance = this;
         d->hostKeyDatabase = QSsh::SshHostKeyDatabasePtr::create();
         const QString keyFilePath = hostKeysFilePath();
-        if (QFileInfo(keyFilePath).exists()) {
+        if (QFileInfo::exists(keyFilePath)) {
             QString error;
             if (!d->hostKeyDatabase->load(keyFilePath, &error))
                 Core::MessageManager::write(error);
@@ -398,34 +406,6 @@ IDevice::ConstPtr DeviceManager::defaultDevice(Core::Id deviceType) const
     return id.isValid() ? find(id) : IDevice::ConstPtr();
 }
 
-void DeviceManager::ensureOneDefaultDevicePerType()
-{
-    foreach (const IDevice::Ptr &device, d->devices) {
-        if (!defaultDevice(device->type()))
-            d->defaultDevices.insert(device->type(), device->id());
-    }
-}
-
-IDevice::Ptr DeviceManager::fromRawPointer(IDevice *device) const
-{
-    foreach (const IDevice::Ptr &devPtr, d->devices) {
-        if (devPtr == device)
-            return devPtr;
-    }
-
-    if (this == instance() && d->clonedInstance)
-        return d->clonedInstance->fromRawPointer(device);
-
-    qWarning("%s: Device not found.", Q_FUNC_INFO);
-    return IDevice::Ptr();
-}
-
-IDevice::ConstPtr DeviceManager::fromRawPointer(const IDevice *device) const
-{
-    // The const_cast is safe, because we convert the Ptr back to a ConstPtr before returning it.
-    return fromRawPointer(const_cast<IDevice *>(device));
-}
-
 QString DeviceManager::hostKeysFilePath()
 {
     return settingsFilePath(QLatin1String("/ssh-hostkeys")).toString();
@@ -451,13 +431,13 @@ public:
     static Core::Id testTypeId() { return "TestType"; }
 private:
     TestDevice(const TestDevice &other) : IDevice(other) {}
-    QString displayType() const { return QLatin1String("blubb"); }
-    IDeviceWidget *createWidget() { return 0; }
-    QList<Core::Id> actionIds() const { return QList<Core::Id>(); }
-    QString displayNameForActionId(Core::Id) const { return QString(); }
-    void executeAction(Core::Id, QWidget *) { }
-    Ptr clone() const { return Ptr(new TestDevice(*this)); }
-    DeviceProcessSignalOperation::Ptr signalOperation() const
+    QString displayType() const override { return QLatin1String("blubb"); }
+    IDeviceWidget *createWidget() override { return 0; }
+    QList<Core::Id> actionIds() const override { return QList<Core::Id>(); }
+    QString displayNameForActionId(Core::Id) const override { return QString(); }
+    void executeAction(Core::Id, QWidget *) override { }
+    Ptr clone() const override { return Ptr(new TestDevice(*this)); }
+    DeviceProcessSignalOperation::Ptr signalOperation() const override
     {
         return DeviceProcessSignalOperation::Ptr();
     }
