@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,44 +9,54 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef TOOLCHAIN_H
-#define TOOLCHAIN_H
+#pragma once
 
 #include "projectexplorer_export.h"
+#include "projectexplorer_global.h"
+
 #include <coreplugin/id.h>
 
+#include <utils/fileutils.h>
+
 #include <QObject>
+#include <QSet>
 #include <QString>
 #include <QVariantMap>
 
-namespace Utils {
-class Environment;
-class FileName;
-}
+#include <functional>
+
+namespace Utils { class Environment; }
 
 namespace ProjectExplorer {
 
 namespace Internal { class ToolChainPrivate; }
 
+namespace Deprecated {
+// Deprecated in 4.3:
+namespace Toolchain {
+enum Language {
+    None = 0,
+    C,
+    Cxx
+};
+QString languageId(Language l);
+} // namespace Toolchain
+} // namespace Deprecated
+
 class Abi;
 class HeaderPath;
 class IOutputParser;
@@ -69,6 +79,8 @@ public:
         AutoDetectionFromSettings
     };
 
+    using Predicate = std::function<bool(const ToolChain *)>;
+
     virtual ~ToolChain();
 
     QString displayName() const;
@@ -77,19 +89,19 @@ public:
     inline bool isAutoDetected() const { return detection() != ManualDetection; }
     Detection detection() const;
 
-    QString id() const;
+    QByteArray id() const;
 
-    virtual QList<Utils::FileName> suggestedMkspecList() const;
+    virtual Utils::FileNameList suggestedMkspecList() const;
     virtual Utils::FileName suggestedDebugger() const;
 
-    virtual QString type() const = 0;
+    Core::Id typeId() const;
     virtual QString typeDisplayName() const = 0;
     virtual Abi targetAbi() const = 0;
+    virtual QList<Abi> supportedAbis() const;
+    virtual QString originalTargetTriple() const { return QString(); }
 
     virtual bool isValid() const = 0;
 
-    virtual QByteArray predefinedMacros(const QStringList &cxxflags) const = 0;
-
     enum CompilerFlag {
         NoFlags = 0,
         StandardCxx11 = 0x1,
@@ -101,47 +113,29 @@ public:
         OpenMP = 0x40,
         ObjectiveC = 0x80,
         StandardCxx14 = 0x100,
-        StandardCxx17 = 0x200
+        StandardCxx17 = 0x200,
+        StandardCxx98 = 0x400,
     };
     Q_DECLARE_FLAGS(CompilerFlags, CompilerFlag)
 
     virtual CompilerFlags compilerFlags(const QStringList &cxxflags) const = 0;
-
-    enum WarningFlag {
-        // General settings
-        WarningsAsErrors,
-        WarningsDefault,
-        WarningsAll,
-        WarningsExtra,
-        WarningsPedantic,
-
-        // Any language
-        WarnUnusedLocals,
-        WarnUnusedParams,
-        WarnUnusedFunctions,
-        WarnUnusedResult,
-        WarnUnusedValue,
-        WarnDocumentation,
-        WarnUninitializedVars,
-        WarnHiddenLocals,
-        WarnUnknownPragma,
-        WarnDeprecated,
-        WarnSignedComparison,
-        WarnIgnoredQualfiers,
-
-        // C++
-        WarnOverloadedVirtual,
-        WarnEffectiveCxx,
-        WarnNonVirtualDestructor
-    };
-    Q_DECLARE_FLAGS(WarningFlags, WarningFlag)
-
     virtual WarningFlags warningFlags(const QStringList &cflags) const = 0;
+
+    // A PredefinedMacrosRunner is created in the ui thread and runs in another thread.
+    using PredefinedMacrosRunner = std::function<QByteArray(const QStringList &cxxflags)>;
+    virtual PredefinedMacrosRunner createPredefinedMacrosRunner() const = 0;
+    virtual QByteArray predefinedMacros(const QStringList &cxxflags) const = 0;
+
+    // A SystemHeaderPathsRunner is created in the ui thread and runs in another thread.
+    using SystemHeaderPathsRunner = std::function<QList<HeaderPath>(const QStringList &cxxflags, const QString &sysRoot)>;
+    virtual SystemHeaderPathsRunner createSystemHeaderPathsRunner() const = 0;
     virtual QList<HeaderPath> systemHeaderPaths(const QStringList &cxxflags,
                                                 const Utils::FileName &sysRoot) const = 0;
     virtual void addToEnvironment(Utils::Environment &env) const = 0;
     virtual QString makeCommand(const Utils::Environment &env) const = 0;
 
+    Core::Id language() const;
+
     virtual Utils::FileName compilerCommand() const = 0;
     virtual IOutputParser *outputParser() const = 0;
 
@@ -155,8 +149,11 @@ public:
     // Make sure to call this function when deriving!
     virtual QVariantMap toMap() const;
     virtual QList<Task> validateKit(const Kit *k) const;
+
+    void setLanguage(Core::Id language);
+
 protected:
-    explicit ToolChain(const QString &id, Detection d);
+    explicit ToolChain(Core::Id typeId, Detection d);
     explicit ToolChain(const ToolChain &);
 
     void toolChainUpdated();
@@ -178,30 +175,28 @@ class PROJECTEXPLORER_EXPORT ToolChainFactory : public QObject
     Q_OBJECT
 
 public:
-    Core::Id id() const { return m_id; }
     QString displayName() const { return m_displayName; }
 
-    virtual QList<ToolChain *> autoDetect();
+    virtual QList<ToolChain *> autoDetect(const QList<ToolChain *> &alreadyKnown);
+    virtual QList<ToolChain *> autoDetect(const Utils::FileName &compilerPath, const Core::Id &language);
 
     virtual bool canCreate();
-    virtual ToolChain *create();
+    virtual ToolChain *create(Core::Id l);
 
     virtual bool canRestore(const QVariantMap &data);
     virtual ToolChain *restore(const QVariantMap &data);
 
-    static QString idFromMap(const QVariantMap &data);
-    static void idToMap(QVariantMap &data, const QString id);
+    static QByteArray idFromMap(const QVariantMap &data);
+    static Core::Id typeIdFromMap(const QVariantMap &data);
     static void autoDetectionToMap(QVariantMap &data, bool detected);
 
+    virtual QSet<Core::Id> supportedLanguages() const = 0;
+
 protected:
-    void setId(Core::Id id) { m_id = id; }
     void setDisplayName(const QString &name) { m_displayName = name; }
 
 private:
     QString m_displayName;
-    Core::Id m_id;
 };
 
 } // namespace ProjectExplorer
-
-#endif // TOOLCHAIN_H
