@@ -23,21 +23,17 @@
 **
 ****************************************************************************/
 
-#ifndef QBSNODES_H
-#define QBSNODES_H
+#pragma once
 
 #include <projectexplorer/projectnodes.h>
 
 #include <qbs.h>
 
-#include <QIcon>
-
 namespace QbsProjectManager {
 namespace Internal {
 
-class FileTreeNode;
+class QbsNodeTreeBuilder;
 class QbsProject;
-class QbsProjectFile;
 
 // ----------------------------------------------------------------------
 // QbsFileNode:
@@ -59,7 +55,7 @@ public:
                   const QString &displayName);
 
 private:
-    QList<ProjectExplorer::ProjectAction> supportedActions(ProjectExplorer::Node *node) const override;
+    bool supportsAction(ProjectExplorer::ProjectAction action, Node *node) const final;
 };
 
 // ---------------------------------------------------------------------------
@@ -74,10 +70,6 @@ public:
     explicit QbsBaseProjectNode(const Utils::FileName &absoluteFilePath);
 
     bool showInSimpleTree() const override;
-
-    QList<ProjectExplorer::ProjectAction> supportedActions(Node *node) const override;
-private:
-    friend class QbsGroupNode;
 };
 
 // --------------------------------------------------------------------
@@ -89,33 +81,16 @@ class QbsGroupNode : public QbsBaseProjectNode
 public:
     QbsGroupNode(const qbs::GroupData &grp, const QString &productPath);
 
-    bool isEnabled() const override;
-    QList<ProjectExplorer::ProjectAction> supportedActions(Node *node) const override;
+    bool supportsAction(ProjectExplorer::ProjectAction action, Node *node) const final;
     bool addFiles(const QStringList &filePaths, QStringList *notAdded = 0) override;
     bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = 0) override;
     bool renameFile(const QString &filePath, const QString &newFilePath) override;
-    void updateQbsGroupData(const qbs::GroupData &grp, const QString &productPath,
-                            bool productWasEnabled, bool productIsEnabled);
 
     qbs::GroupData qbsGroupData() const { return m_qbsGroupData; }
 
-    QString productPath() const;
-
-    // group can be invalid
-    static void setupFiles(FolderNode *root, const qbs::GroupData &group, const QStringList &files,
-                           const QString &productPath, bool updateExisting);
-
 private:
-    static void setupFolder(ProjectExplorer::FolderNode *folder,
-                            const QHash<QString, ProjectExplorer::FileType> &fileTypeHash,
-                            const FileTreeNode *subFileTree, const QString &baseDir,
-                            bool updateExisting);
-    static ProjectExplorer::FileType fileType(const qbs::SourceArtifact &artifact);
-
     qbs::GroupData m_qbsGroupData;
     QString m_productPath;
-
-    static QIcon m_groupIcon;
 };
 
 // --------------------------------------------------------------------
@@ -125,25 +100,20 @@ private:
 class QbsProductNode : public QbsBaseProjectNode
 {
 public:
-    explicit QbsProductNode(const qbs::Project &project, const qbs::ProductData &prd);
+    explicit QbsProductNode(const qbs::ProductData &prd);
 
-    bool isEnabled() const override;
     bool showInSimpleTree() const override;
-    QList<ProjectExplorer::ProjectAction> supportedActions(Node *node) const override;
+    bool supportsAction(ProjectExplorer::ProjectAction action, Node *node) const final;
     bool addFiles(const QStringList &filePaths, QStringList *notAdded = 0) override;
     bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = 0) override;
     bool renameFile(const QString &filePath, const QString &newFilePath) override;
 
-    void setQbsProductData(const qbs::Project &project, const qbs::ProductData prd);
     const qbs::ProductData qbsProductData() const { return m_qbsProductData; }
 
     QList<ProjectExplorer::RunConfiguration *> runConfigurations() const override;
 
 private:
-    QbsGroupNode *findGroupNode(const QString &name);
-
-    qbs::ProductData m_qbsProductData;
-    static QIcon m_productIcon;
+    const qbs::ProductData m_qbsProductData;
 };
 
 // ---------------------------------------------------------------------------
@@ -153,26 +123,19 @@ private:
 class QbsProjectNode : public QbsBaseProjectNode
 {
 public:
-    explicit QbsProjectNode(const Utils::FileName &absoluteFilePath);
-    ~QbsProjectNode() override;
+    explicit QbsProjectNode(const Utils::FileName &projectDirectory);
 
     virtual QbsProject *project() const;
     const qbs::Project qbsProject() const;
     const qbs::ProjectData qbsProjectData() const { return m_projectData; }
 
     bool showInSimpleTree() const override;
-
-protected:
-    void update(const qbs::Project &qbsProject, const qbs::ProjectData &prjData);
+    void setProjectData(const qbs::ProjectData &data); // FIXME: Needed?
 
 private:
-    void ctor();
-
-    QbsProductNode *findProductNode(const QString &uniqueName);
-    QbsProjectNode *findProjectNode(const QString &name);
-
-    static QIcon m_projectIcon;
     qbs::ProjectData m_projectData;
+
+    friend class QbsNodeTreeBuilder;
 };
 
 // --------------------------------------------------------------------
@@ -184,20 +147,12 @@ class QbsRootProjectNode : public QbsProjectNode
 public:
     explicit QbsRootProjectNode(QbsProject *project);
 
-    using QbsProjectNode::update;
-    void update();
-
     QbsProject *project() const  override { return m_project; }
 
 private:
-    QStringList unreferencedBuildSystemFiles(const qbs::Project &p) const;
-
     QbsProject *const m_project;
-    ProjectExplorer::FolderNode *m_buildSystemFiles;
 };
 
 
 } // namespace Internal
 } // namespace QbsProjectManager
-
-#endif // QBSNODES_H
