@@ -48,6 +48,7 @@ QT_END_NAMESPACE
 
 namespace ProjectExplorer {
 class Kit;
+class Project;
 }
 
 namespace QmlDesigner {
@@ -90,6 +91,7 @@ public:
                         const NodeAbstractProperty &oldPropertyParent,
                         AbstractView::PropertyChangeFlags propertyChange) override;
     void rootNodeTypeChanged(const QString &type, int majorVersion, int minorVersion) override;
+    void nodeTypeChanged(const ModelNode& node, const TypeName &type, int majorVersion, int minorVersion) override;
     void fileUrlChanged(const QUrl &oldUrl, const QUrl &newUrl) override;
     void nodeIdChanged(const ModelNode& node, const QString& newId, const QString& oldId) override;
     void nodeOrderChanged(const NodeListProperty &listProperty, const ModelNode &movedNode, int oldIndex) override;
@@ -127,13 +129,10 @@ public:
     QImage statePreviewImage(const ModelNode &stateNode) const;
 
     void setKit(ProjectExplorer::Kit *kit);
+    void setProject(ProjectExplorer::Project *project);
 
     void sendToken(const QString &token, int number, const QVector<ModelNode> &nodeVector);
 
-signals:
-    void qmlPuppetCrashed();
-    void qmlPuppetError(const QString &errorMessage);
-
 protected:
     void timerEvent(QTimerEvent *event) override;
 
@@ -184,8 +183,8 @@ private: // functions
     void restartProcess();
     void delayedRestartProcess();
 
-private slots:
-    void handleChrash();
+private:
+    void handleCrash();
 
 private: //variables
     NodeInstance m_rootNodeInstance;
@@ -199,7 +198,8 @@ private: //variables
     QImage m_baseStatePreviewImage;
     QTime m_lastCrashTime;
     NodeInstanceServerInterface::RunModus m_runModus;
-    ProjectExplorer::Kit *m_currentKit;
+    ProjectExplorer::Kit *m_currentKit = nullptr;
+    ProjectExplorer::Project *m_currentProject = nullptr;
     int m_restartProcessTimerId;
 };
 
