@@ -34,6 +34,7 @@
 #include <model.h>
 #include <modelnode.h>
 #include <metainfo.h>
+#include <rewriterview.h>
 
 #include "abstractproperty.h"
 #include "variantproperty.h"
@@ -103,7 +104,6 @@ NodeInstanceView::NodeInstanceView(QObject *parent, NodeInstanceServerInterface:
         : AbstractView(parent),
           m_baseStatePreviewImage(QSize(100, 100), QImage::Format_ARGB32),
           m_runModus(runModus),
-          m_currentKit(0),
           m_restartProcessTimerId(0)
 {
     m_baseStatePreviewImage.fill(0xFFFFFF);
@@ -151,9 +151,10 @@ bool isSkippedNode(const ModelNode &node)
 void NodeInstanceView::modelAttached(Model *model)
 {
     AbstractView::modelAttached(model);
-    m_nodeInstanceServer = new NodeInstanceServerProxy(this, m_runModus, m_currentKit);
+    auto server = new NodeInstanceServerProxy(this, m_runModus, m_currentKit, m_currentProject);
+    m_nodeInstanceServer = server;
     m_lastCrashTime.start();
-    connect(m_nodeInstanceServer.data(), SIGNAL(processCrashed()), this, SLOT(handleChrash()));
+    connect(server, &NodeInstanceServerProxy::processCrashed, this, &NodeInstanceView::handleCrash);
 
     if (!isSkippedRootNode(rootModelNode()))
         nodeInstanceServer()->createScene(createCreateSceneCommand());
@@ -181,30 +182,36 @@ void NodeInstanceView::modelAboutToBeDetached(Model * model)
     AbstractView::modelAboutToBeDetached(model);
 }
 
-void NodeInstanceView::handleChrash()
+void NodeInstanceView::handleCrash()
 {
     int elaspsedTimeSinceLastCrash = m_lastCrashTime.restart();
-
-    if (elaspsedTimeSinceLastCrash > 2000)
+    int forceRestartTime = 2000;
+#ifdef QT_DEBUG
+    forceRestartTime = 4000;
+#endif
+    if (elaspsedTimeSinceLastCrash > forceRestartTime)
         restartProcess();
     else
-        emit qmlPuppetCrashed();
+        emitDocumentMessage(tr("Qt Quick emulation layer crashed."));
 
     emitCustomNotification(QStringLiteral("puppet crashed"));
 }
 
-
-
 void NodeInstanceView::restartProcess()
 {
+    if (rootNodeInstance().isValid())
+        rootNodeInstance().setError({});
+    emitInstanceErrorChange({});
+
     if (m_restartProcessTimerId)
         killTimer(m_restartProcessTimerId);
 
     if (model()) {
         delete nodeInstanceServer();
 
-        m_nodeInstanceServer = new NodeInstanceServerProxy(this, m_runModus, m_currentKit);
-        connect(m_nodeInstanceServer.data(), SIGNAL(processCrashed()), this, SLOT(handleChrash()));
+        auto server = new NodeInstanceServerProxy(this, m_runModus, m_currentKit, m_currentProject);
+        m_nodeInstanceServer = server;
+        connect(server, &NodeInstanceServerProxy::processCrashed, this, &NodeInstanceView::handleCrash);
 
         if (!isSkippedRootNode(rootModelNode()))
             nodeInstanceServer()->createScene(createCreateSceneCommand());
@@ -362,6 +369,11 @@ void NodeInstanceView::rootNodeTypeChanged(const QString &/*type*/, int /*majorV
     restartProcess();
 }
 
+void NodeInstanceView::nodeTypeChanged(const ModelNode &, const TypeName &, int, int)
+{
+    restartProcess();
+}
+
 void NodeInstanceView::bindingPropertiesChanged(const QList<BindingProperty>& propertyList, PropertyChangeFlags /*propertyChange*/)
 {
     nodeInstanceServer()->changePropertyBindings(createChangeBindingCommand(propertyList));
@@ -841,6 +853,44 @@ CreateSceneCommand NodeInstanceView::createCreateSceneCommand()
     foreach (const Import &import, model()->imports())
         importVector.append(AddImportContainer(import.url(), import.file(), import.version(), import.alias(), import.importPaths()));
 
+    QVector<MockupTypeContainer> mockupTypesVector;
+
+    for (const CppTypeData &cppTypeData : model()->rewriterView()->getCppTypes()) {
+        const QString versionString = cppTypeData.versionString;
+        int majorVersion = -1;
+        int minorVersion = -1;
+
+        if (versionString.contains(QStringLiteral("."))) {
+            const QStringList splittedString = versionString.split(QStringLiteral("."));
+            majorVersion = splittedString.first().toInt();
+            minorVersion = splittedString.last().toInt();
+        }
+
+        bool isItem = false;
+
+        if (!cppTypeData.isSingleton) { /* Singletons only appear on the right hand sides of bindings and create just warnings. */
+            const TypeName typeName = cppTypeData.typeName.toUtf8();
+            const QString uri = cppTypeData.importUrl;
+
+            NodeMetaInfo metaInfo = model()->metaInfo(uri.toUtf8() + "." + typeName);
+
+            if (metaInfo.isValid())
+                isItem = metaInfo.isGraphicalItem();
+
+            MockupTypeContainer mockupType(typeName, uri, majorVersion, minorVersion, isItem);
+
+            mockupTypesVector.append(mockupType);
+        } else { /* We need a type for the signleton import */
+            const TypeName typeName = cppTypeData.typeName.toUtf8() + "Mockup";
+            const QString uri = cppTypeData.importUrl;
+
+            MockupTypeContainer mockupType(typeName, uri, majorVersion, minorVersion, isItem);
+
+            mockupTypesVector.append(mockupType);
+        }
+    }
+
+
     return CreateSceneCommand(instanceContainerList,
                               reparentContainerList,
                               idContainerList,
@@ -848,6 +898,7 @@ CreateSceneCommand NodeInstanceView::createCreateSceneCommand()
                               bindingContainerList,
                               auxiliaryContainerVector,
                               importVector,
+                              mockupTypesVector,
                               model()->fileUrl());
 }
 
@@ -1084,6 +1135,8 @@ void NodeInstanceView::pixmapChanged(const PixmapChangedCommand &command)
         }
     }
 
+    m_nodeInstanceServer->benchmark(Q_FUNC_INFO + QString::number(renderImageChangeSet.count()));
+
     if (!renderImageChangeSet.isEmpty())
         emitInstancesRenderImageChanged(renderImageChangeSet.toList().toVector());
 }
@@ -1113,6 +1166,8 @@ void NodeInstanceView::informationChanged(const InformationChangedCommand &comma
 
     QMultiHash<ModelNode, InformationName> informationChangeHash = informationChanged(command.informations());
 
+    m_nodeInstanceServer->benchmark(Q_FUNC_INFO + QString::number(informationChangeHash.count()));
+
     if (!informationChangeHash.isEmpty())
         emitInstanceInformationsChange(informationChangeHash);
 }
@@ -1133,6 +1188,14 @@ void NodeInstanceView::setKit(ProjectExplorer::Kit *newKit)
     }
 }
 
+void NodeInstanceView::setProject(ProjectExplorer::Project *project)
+{
+    if (m_currentProject != project) {
+        m_currentProject = project;
+        restartProcess();
+    }
+}
+
 void NodeInstanceView::statePreviewImagesChanged(const StatePreviewImageChangedCommand &command)
 {
     if (!model())
@@ -1169,6 +1232,8 @@ void NodeInstanceView::componentCompleted(const ComponentCompletedCommand &comma
             nodeVector.append(modelNodeForInternalId(instanceId));
     }
 
+    m_nodeInstanceServer->benchmark(Q_FUNC_INFO + QString::number(nodeVector.count()));
+
     if (!nodeVector.isEmpty())
         emitInstancesCompleted(nodeVector);
 }
@@ -1184,10 +1249,9 @@ void NodeInstanceView::childrenChanged(const ChildrenChangedCommand &command)
     foreach (qint32 instanceId, command.childrenInstances()) {
         if (hasInstanceForId(instanceId)) {
             NodeInstance instance = instanceForId(instanceId);
-            if (!instance.directUpdates()) {
+            if (instance.parentId() == -1 || !instance.directUpdates())
                 instance.setParentId(command.parentInstanceId());
-                childNodeVector.append(instance.modelNode());
-            }
+            childNodeVector.append(instance.modelNode());
         }
     }
 
@@ -1212,14 +1276,14 @@ void NodeInstanceView::token(const TokenCommand &command)
             nodeVector.append(modelNodeForInternalId(instanceId));
     }
 
-
     emitInstanceToken(command.tokenName(), command.tokenNumber(), nodeVector);
 }
 
 void NodeInstanceView::debugOutput(const DebugOutputCommand & command)
 {
+    DocumentMessage error(tr("Qt Quick emulation layer crashed."));
     if (command.instanceIds().isEmpty()) {
-        qmlPuppetError(command.text()); // TODO: connect that somewhere to show that to the user
+        emitDocumentMessage(command.text());
     } else {
         QVector<qint32> instanceIdsWithChangedErrors;
         foreach (qint32 instanceId, command.instanceIds()) {
@@ -1228,7 +1292,7 @@ void NodeInstanceView::debugOutput(const DebugOutputCommand & command)
                 if (instance.setError(command.text()))
                     instanceIdsWithChangedErrors.append(instanceId);
             } else {
-                qmlPuppetError(command.text()); // TODO: connect that somewhere to show that to the user
+                emitDocumentMessage(command.text());
             }
         }
         emitInstanceErrorChange(instanceIdsWithChangedErrors);
