@@ -28,7 +28,7 @@
 #include "baseannotationhighlighter.h"
 #include "basevcseditorfactory.h"
 #include "vcsbaseplugin.h"
-#include "vcsbaseeditorparameterwidget.h"
+#include "vcsbaseeditorconfig.h"
 #include "vcscommand.h"
 
 #include <coreplugin/icore.h>
@@ -156,13 +156,7 @@ VcsBaseEditor::VcsBaseEditor()
 
 void VcsBaseEditor::finalizeInitialization()
 {
-    auto widget = qobject_cast<VcsBaseEditorWidget *>(editorWidget());
-    QTC_ASSERT(widget, return);
-    // Pass on signals.
-    connect(widget, &VcsBaseEditorWidget::describeRequested,
-            this, &VcsBaseEditor::describeRequested);
-    connect(widget, &VcsBaseEditorWidget::annotateRevisionRequested,
-            this, &VcsBaseEditor::annotateRevisionRequested);
+    QTC_CHECK(qobject_cast<VcsBaseEditorWidget *>(editorWidget()));
 }
 
 // ----------- VcsBaseEditorPrivate
@@ -553,43 +547,34 @@ public:
     QComboBox *entriesComboBox();
 
     TextEditorWidget *q;
-    const VcsBaseEditorParameters *m_parameters;
+    const VcsBaseEditorParameters *m_parameters = nullptr;
 
     QString m_workingDirectory;
 
     QRegExp m_diffFilePattern;
     QRegExp m_logEntryPattern;
     QList<int> m_entrySections; // line number where this section starts
-    int m_cursorLine;
+    int m_cursorLine = -1;
+    int m_firstLineNumber = -1;
     QString m_annotateRevisionTextFormat;
     QString m_annotatePreviousRevisionTextFormat;
     QString m_copyRevisionTextFormat;
-    bool m_fileLogAnnotateEnabled;
-    VcsBaseEditorParameterWidget *m_configurationWidget;
-    bool m_mouseDragging;
+    bool m_configurationAdded = false;
     QList<AbstractTextCursorHandler *> m_textCursorHandlers;
     QPointer<VcsCommand> m_command;
-    QObject *m_describeReceiver;
-    const char *m_describeSlot;
-    Utils::ProgressIndicator *m_progressIndicator;
+    VcsBaseEditorWidget::DescribeFunc m_describeFunc = nullptr;
+    Utils::ProgressIndicator *m_progressIndicator = nullptr;
+    bool m_fileLogAnnotateEnabled = false;
+    bool m_mouseDragging = false;
 
 private:
-    QComboBox *m_entriesComboBox;
+    QComboBox *m_entriesComboBox = nullptr;
 };
 
 VcsBaseEditorWidgetPrivate::VcsBaseEditorWidgetPrivate(VcsBaseEditorWidget *editorWidget)  :
     q(editorWidget),
-    m_parameters(0),
-    m_cursorLine(-1),
     m_annotateRevisionTextFormat(VcsBaseEditorWidget::tr("Annotate \"%1\"")),
-    m_copyRevisionTextFormat(VcsBaseEditorWidget::tr("Copy \"%1\"")),
-    m_fileLogAnnotateEnabled(false),
-    m_configurationWidget(0),
-    m_mouseDragging(false),
-    m_describeReceiver(0),
-    m_describeSlot(0),
-    m_progressIndicator(0),
-    m_entriesComboBox(0)
+    m_copyRevisionTextFormat(VcsBaseEditorWidget::tr("Copy \"%1\""))
 {
     m_textCursorHandlers.append(new ChangeTextCursorHandler(editorWidget));
     m_textCursorHandlers.append(new UrlTextCursorHandler(editorWidget));
@@ -690,17 +675,45 @@ QString VcsBaseEditorWidget::fileNameForLine(int line) const
     return source();
 }
 
-void VcsBaseEditorWidget::setDescribeSlot(QObject *describeReceiver, const char *describeSlot)
+int VcsBaseEditorWidget::firstLineNumber() const
 {
-    d->m_describeReceiver = describeReceiver;
-    d->m_describeSlot = describeSlot;
+    return d->m_firstLineNumber;
+}
+
+void VcsBaseEditorWidget::setFirstLineNumber(int firstLineNumber)
+{
+    d->m_firstLineNumber = firstLineNumber;
+}
+
+QString VcsBaseEditorWidget::lineNumber(int blockNumber) const
+{
+    if (d->m_firstLineNumber > 0)
+        return QString::number(d->m_firstLineNumber + blockNumber);
+    return TextEditorWidget::lineNumber(blockNumber);
+}
+
+int VcsBaseEditorWidget::lineNumberDigits() const
+{
+    if (d->m_firstLineNumber <= 0)
+        return TextEditorWidget::lineNumberDigits();
+
+    int digits = 2;
+    int max = qMax(1, d->m_firstLineNumber + blockCount());
+    while (max >= 100) {
+        max /= 10;
+        ++digits;
+    }
+    return digits;
+}
+
+void VcsBaseEditorWidget::setDescribeFunc(DescribeFunc describeFunc)
+{
+    d->m_describeFunc = describeFunc;
 }
 
 void VcsBaseEditorWidget::finalizeInitialization()
 {
-    if (d->m_describeReceiver)
-        connect(this, SIGNAL(describeRequested(QString,QString)), d->m_describeReceiver, d->m_describeSlot);
-
+    connect(this, &VcsBaseEditorWidget::describeRequested, this, d->m_describeFunc);
     init();
 }
 
@@ -1199,10 +1212,15 @@ DiffChunk VcsBaseEditorWidget::diffChunk(QTextCursor cursor) const
 void VcsBaseEditorWidget::reportCommandFinished(bool ok, int exitCode, const QVariant &data)
 {
     Q_UNUSED(exitCode);
-    Q_UNUSED(data);
 
-    if (!ok)
+    hideProgressIndicator();
+    if (!ok) {
         textDocument()->setPlainText(tr("Failed to retrieve data."));
+    } else if (data.type() == QVariant::Int) {
+        const int line = data.toInt();
+        if (line >= 0)
+            gotoLine(line);
+    }
 }
 
 const VcsBaseEditorParameters *VcsBaseEditor::findType(const VcsBaseEditorParameters *array,
@@ -1239,7 +1257,7 @@ static QTextCodec *findProjectCodec(const QString &dir)
                     return codec;
                 }
     }
-    return 0;
+    return nullptr;
 }
 
 QTextCodec *VcsBaseEditor::getCodec(const QString &source)
@@ -1362,20 +1380,14 @@ QString VcsBaseEditor::getTitleId(const QString &workingDirectory,
     return rc;
 }
 
-bool VcsBaseEditorWidget::setConfigurationWidget(VcsBaseEditorParameterWidget *w)
+void VcsBaseEditorWidget::setConfigurationAdded()
 {
-    if (d->m_configurationWidget)
-        return false;
-
-    d->m_configurationWidget = w;
-    insertExtraToolBarWidget(TextEditorWidget::Right, w);
-
-    return true;
+    d->m_configurationAdded = true;
 }
 
-VcsBaseEditorParameterWidget *VcsBaseEditorWidget::configurationWidget() const
+bool VcsBaseEditorWidget::configurationAdded() const
 {
-    return d->m_configurationWidget;
+    return d->m_configurationAdded;
 }
 
 void VcsBaseEditorWidget::setCommand(VcsCommand *command)
@@ -1385,11 +1397,10 @@ void VcsBaseEditorWidget::setCommand(VcsCommand *command)
         hideProgressIndicator();
     }
     d->m_command = command;
-    if (d->m_command) {
+    if (command) {
         d->m_progressIndicator = new Utils::ProgressIndicator(Utils::ProgressIndicator::Large);
         d->m_progressIndicator->attachToWidget(this);
-        connect(d->m_command.data(), &VcsCommand::finished,
-                this, &VcsBaseEditorWidget::hideProgressIndicator);
+        connect(command, &VcsCommand::finished, this, &VcsBaseEditorWidget::reportCommandFinished);
         QTimer::singleShot(100, this, &VcsBaseEditorWidget::showProgressIndicator);
     }
 }
