@@ -43,8 +43,8 @@
 #include <language/filetags.h>
 #include <plugins/scanner/scanner.h>
 
-#include <QHash>
-#include <QString>
+#include <QtCore/qhash.h>
+#include <QtCore/qstring.h>
 
 QT_BEGIN_NAMESPACE
 class QLibrary;
@@ -60,6 +60,7 @@ public:
     ~ScannerPluginManager();
     static ScannerPluginManager *instance();
     static QList<ScannerPlugin *> scannersForFileTag(const FileTag &fileTag);
+    void loadPlugins(ScannerPlugin **plugins);
     void loadPlugins(const QStringList &paths, const Logger &logger);
 
 private:
