@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef SHELL_H
-#define SHELL_H
+#pragma once
 
 #include <QObject>
 #include <QSharedPointer>
@@ -50,7 +49,7 @@ public:
 
     void run();
 
-private slots:
+private:
     void handleConnected();
     void handleConnectionError();
     void handleRemoteStdout();
@@ -60,10 +59,7 @@ private slots:
     void handleShellStarted();
     void handleStdin();
 
-private:
     QSsh::SshConnection *m_connection;
     QSharedPointer<QSsh::SshRemoteProcess> m_shell;
     QFile * const m_stdin;
 };
-
-#endif // SHELL_H
